;  Script-Fu for The GIMP -- an image manipulation program
;
;  LILO-SPLASH-SCREEN 	V 1.1
;
;  This script requires an image and a current selection 
;   within which is the region of interest where the 
;   graphical boot menu will be displayed.
;
;  Additional colors for "Menu Item Text" and "Selected Menu
;   Item Text" can be specified as well as two color values
;   used for selection manipulation (Menu Bar ADD/SUB)
;
;  V 1.0 Basic Functionality
;  V 1.1 Fix to compensate for missing palette colors
;        (major rewrite of color selection/padding)
;
;  ---------------
;  Copyright (C) 2001 Herbert Ptzl
;  herbert@13thfloor.at
;
;   if you think you should contact me, ask yourself again,
;   and, if your decision still stands, feel free to do so.
;  ---------------
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;


(define (script-fu-lilo-splash-screen 
				img drawable
				num-colors 
				add-color sub-color
				text-color sel-color)
  (let* (
      (dither-type 3)
 
      (sel-bounds (gimp-selection-bounds img))
      (sel-offx (cadr sel-bounds))
      (sel-offy (caddr sel-bounds))
      (sel-width (- (cadr (cddr sel-bounds)) sel-offx))
      (sel-height (- (caddr (cddr sel-bounds)) sel-offy))

      (sel-img (car (gimp-image-new sel-width sel-height RGB)))
      (sel-lay (car (gimp-layer-new sel-img sel-width sel-height
  			  RGB_IMAGE "Selection" 100 NORMAL)))

      (pal-img (car (gimp-channel-ops-duplicate img)))
      (pal-lay (car (gimp-image-flatten pal-img)))
    )	
			
    (gimp-image-undo-disable img)
    (gimp-palette-set-background '(0 0 0))

;
;   make separate image from selection
;    

    (gimp-image-add-layer sel-img sel-lay 0)
    (gimp-drawable-fill sel-lay 3)
    (gimp-selection-all sel-img)
   
    (gimp-edit-copy drawable)
    (set! sel-float (car (gimp-edit-paste sel-lay 0)))

;
;   make indexed with "Num Colors" ...
;

    (gimp-convert-indexed sel-img dither-type 
    	0 num-colors 0 0 "")

;
;   remove selection from original and
;   convert to reduced indexed palette
;

    (gimp-edit-clear pal-lay 1)
    (gimp-convert-indexed pal-img dither-type 
    	0 (- 254 (* num-colors 2)) 0 0 "")

;
;   some definitions used for the next step
;   mainly fillup, clamping and conversion 
;

    (define (fillup a m)
       (if (< (/ (length a) 3) m) 
       	   (fillup (bytes-append a "\0\0\0") m) a)
    )
    
    
    (define (cma v)
      (if (< v 0) (+ 256 v) v)  
    )

    (define (cmb v)
      (if (< v 0) 0 (if (< v 128) v (- v 256)))  
    )

    (define (cmod v d)
      (cmb (if (< (cma v) (- 0 d)) 0 
        (if (< (cma v) (- 255 d)) (+ (cma v) d) 255)))
    )

;
;   build new color maps
;

    (set! cmaps 
    	(fillup (car (cdr (gimp-image-get-cmap sel-img)))
	 num-colors))
    (set! cmapr 
    	(fillup (car (cdr (gimp-image-get-cmap sel-img)))
	 num-colors))
  
    (let* (
        (index 0)
	(rdif (- (car add-color) (car sub-color)))
	(gdif (- (cadr add-color) (cadr sub-color)))
	(bdif (- (caddr add-color) (caddr sub-color)))	
      )
      (while (< index num-colors)
        (let* (
	  (rind (* (- (- num-colors index) 1) 3))
	  (cind (* index 3))
	  )
          (aset cmapr (+ rind 0) 
	    (cmod (aref cmaps (+ cind 0)) rdif)) 
          (aset cmapr (+ rind 1) 
	    (cmod (aref cmaps (+ cind 1)) gdif)) 
          (aset cmapr (+ rind 2)
	    (cmod (aref cmaps (+ cind 2)) bdif)) 
	  (set! index (+ index 1))
        )
      )
    )
   
;
;   create picture/text/sel color  map ...
;   

    (let* (
        (pic-colors (- 254 (* num-colors 2)))
      )
      (set! cmapp (bytes-append "\0\0\0" 
    	(fillup (car (cdr (gimp-image-get-cmap pal-img)))
	 pic-colors) "\0\0\0"))
 
      (aset cmapp (+ (* 3 0) 0) (car text-color))
      (aset cmapp (+ (* 3 0) 1) (cadr text-color))
      (aset cmapp (+ (* 3 0) 2) (caddr text-color))
      (aset cmapp (+ (* 3 (+ pic-colors 1)) 0) (car sel-color))
      (aset cmapp (+ (* 3 (+ pic-colors 1)) 1) (cadr sel-color))
      (aset cmapp (+ (* 3 (+ pic-colors 1)) 2) (caddr sel-color))
    )	

;
;   adapt image to new color map
;

    (let* (
	(cmapn (bytes-append cmaps cmapp cmapr))
      )
      (gimp-image-set-cmap pal-img (* 3 num-colors) cmaps)
      (gimp-edit-copy sel-float)
      (gimp-edit-paste pal-lay 0)
      
      (gimp-image-set-cmap pal-img (* 3 256) cmapn)
      (gimp-selection-invert img)
      (gimp-edit-copy drawable)
      (gimp-floating-sel-anchor 
      	(car (gimp-edit-paste pal-lay 0)))
      (gimp-selection-invert img)
      (gimp-selection-all pal-img)
    )

;
;   free intermediate images 
;

    (gimp-image-delete sel-img)
    (gimp-display-new pal-img)
    
    (gimp-image-undo-enable img)))

(script-fu-register "script-fu-lilo-splash-screen"
		    "<Image>/Script-Fu/Utils/LILO Splash Screen"
		    "prepare image for use with LILO."
		    "Herbert Poetzl"
		    "Herbert Poetzl"
		    "2001"
		    "RGB* GRAY*"
		    SF-IMAGE "Image" 0
		    SF-DRAWABLE "Drawable" 0
		    SF-ADJUSTMENT "Num Colors" '(16 0 255 1 10 0 1)
		    SF-COLOR "Add Color" '(0 0 0)
		    SF-COLOR "Sub Color" '(32 32 32)
		    SF-COLOR "Text Color" '(216 216 216)
		    SF-COLOR "Select Color" '(255 255 255))
